# -*- coding: utf-8 -*-
# $Id: events.py 106368 2009-12-10 12:02:45Z mborch $
"""Zope 3 style event handlers"""

from zope.component import adapter
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from Products.CMFCore.interfaces import IContentish
from Products.Collage.interfaces import ICollageAlias

@adapter(ICollageAlias, IObjectModifiedEvent)
def updateCollageAliasLayout(context, event):
    """Updating alias layout on alias changed
    """
    target = context.get_target()
    if target:
        layout = target.getLayout()
        context.setLayout(layout)

@adapter(IContentish, IObjectModifiedEvent)
def reindexOnModify(content, event):
    """Collage subcontent change triggers Collage reindexing
    """
    helper = content.restrictedTraverse('@@collage_helper')
    collage = helper.getCollageObject()
    if collage:
        # Change done in a Collage subobject
        collage.reindexObject()



# -*- coding: utf-8 -*-
# $Id: interfaces.py 145424 2010-10-04 12:25:02Z khink $

from zope import interface
from zope import schema
from zope.viewlet.interfaces import IViewletManager
from zope.i18nmessageid import MessageFactory
from Products.Collage.config import I18N_DOMAIN

_ = MessageFactory(I18N_DOMAIN)

class ICollage(interface.Interface):
    pass

class ICollageRow(interface.Interface):
    pass

class ICollageColumn(interface.Interface):
    pass

class ICollageAlias(interface.Interface):
    pass

class IDynamicViewManager(interface.Interface):
    pass

class ICollageBrowserLayer(interface.Interface):
    """Collage browser layer. Views registered with this layer
    are available to objects inside a collage."""
    pass

class ICollageBrowserLayerType(interface.interfaces.IInterface):
    """Marker interface for Collage theme-specific layers
    """
    pass

class ICollageEditLayer(interface.Interface):
    """Collage edit layer."""
    pass

class IContentMenu(IViewletManager):
    """Interface for the content-menu viewlet manager."""
    pass

class IPortletSkin(interface.Interface):
    """Interface for skinable portlets views."""
    pass

class ICollageSiteOptions(interface.Interface):

    ref_browser_empty = schema.Bool(
        title=_(u'label_ref_browser_empty',
                default=u"Open reference Browser without initial item listing"),
        description=_(u'help_ref_browser_empty',
                      default=u"Use this if you have lots of items in your "
                              u"containers."),
        default=False,
        required=True)

    ref_browser_types = schema.Bool(
        title=_(u'label_ref_browser_types',
                default=u"Provide type filter in reference browser"),
        description=_(u'help_ref_browser_types',
                      default=u"When set to True, the reference browser shows "
                              u"up a content type filter."),
        default=False,
        required=True)

    use_whitelist = schema.Bool(
        title=_(u'label_use_whitelist', default=u"Use types whitelist"),
        description=_(u'help_use_whitelist',
                      default=u"Only types in below whitelist can be added in "
                            u"a Collage."),
        default=False,
        required=True)

    types_whitelist = schema.Tuple(
        title=_(u'label_types_whitelist', default=u"Types whitelist"),
        description=_(u'help_types_whitelist',
                      default=u"Select item types that can be added in a "
                              u"Collage object."),
        required=False,
        missing_value=tuple(),
        value_type=schema.Choice(
            vocabulary='collage.vocabularies.CollageUserFriendlyTypes')
        )

    alias_whitelist = schema.Tuple(
        title=_(u'label_alias_whitelist', default=u"Alias whitelist"),
        description=_(u'help_alias_whitelist',
                      default=u"Select item types that can be aliased in a "
                            u"Collage object."),
        required=False,
        missing_value=tuple(),
        value_type=schema.Choice(
            vocabulary='collage.vocabularies.CollageUserFriendlyTypes')
        )

    batch_size = schema.Int(
        title=_(u'label_batch_size', default=u"Number of Columns"),
        description=_(u'help_batch_size', 
            default=u"If the number of Columns in a row exceeds this number, "
                """"Collage will put these in a new "batch". """
                "The new batch will be displayed below the first batch, "
                "but it's technically still the same row. "),
        required=False,
        default=3,
        )

    def enabledType(portal_type):
        """True if portal type is enabled for adding in a Collage."""

    def enabledAlias(portal_type):
        """True if portal type is enabled for alias in a Collage"""



# -*- coding: utf-8 -*-
# $Id: upgrades.py 146142 2010-10-13 16:39:02Z glenfant $
"""Misc GenericSetup upgrade steps"""

# Warning, the various upgrade handlers here must be coded in
# a defensive way. This means that the changes each handler does
# may have already done. The handler must behave accordingly.

from Products.Collage.utilities import getPortal, IfInstalled

PROFILE_NAME = 'profile-Products.Collage:default'

safety_belt = IfInstalled()

@safety_belt
def runTypesStepOnly(setuptool):
    """We upgrade our types only
    """
    setuptool.runImportStepFromProfile(PROFILE_NAME, 'typeinfo', run_dependencies=True)
    return


@safety_belt
def updateJSRegistry(setuptool):
    """Javascript moved from skins to resources
    """
    OLDID = 'collage.js'
    NEWID = '++resource++collage-resources/collage.js'
    jstool = getPortal().portal_javascripts
    all_rscids = jstool.getResourceIds()
    if (OLDID in all_rscids) and (NEWID not in all_rscids):
        jstool.renameResource(OLDID, NEWID)
    return


@safety_belt
def removeSkinsLayer(setuptool):
    """Collage doesn't require a CMF skins layer anymore
    """
    LAYERNAME = 'Collage'
    skinstool = getPortal().portal_skins
    # Unfortunately, there's no easy way to remove a layer from all skins

    skinnames = skinstool.selections.keys()
    for name in skinnames:
        layers = skinstool.selections[name]
        layers = [l.strip() for l in layers.split(',')
                  if l.strip() != LAYERNAME]
        layers = ','.join(layers)
        skinstool.selections[name] = layers
    if LAYERNAME in skinstool.objectIds():
        skinstool._delObject(LAYERNAME)
    return


@safety_belt
def addControlPanel(setuptool):
    """Add Collage control panel resources
    """
    for step in ('propertiestool', 'controlpanel', 'action-icons'):
        setuptool.runImportStepFromProfile(PROFILE_NAME, step, run_dependencies=False)
    return


@safety_belt
def addAliasWhitelistProperty(setuptool):
    """Add Alias whitelist control panel property
    """
    propsheet = getPortal().portal_properties.collage_properties
    # Default: same value as types whitelist
    default_value = propsheet.getProperty('types_whitelist')
    if propsheet.getProperty('alias_whitelist'):
        # property already exists; upgrade step has already been run
        # apparently; upgrade steps should not fail when run a second
        # time, so we return here.
        return
    propsheet.manage_addProperty('alias_whitelist', default_value, 'lines')
    return


@safety_belt
def upgradeTo1_3_0(setuptool):
    """Lots of changes in setup in 1.3.0
    """
    # We must delete ftis since something whoes in the 'typeinfo' step
    # FIXME: is this a GS bug or feature?
    portal_types = getPortal().portal_types
    for portal_type in ('Collage', 'CollageRow', 'CollageColumn', 'CollageAlias'):
        portal_types._delObject(portal_type)

    for step in ('typeinfo', 'cssregistry', 'actions', 'action-icons', 'jsregistry'):
        setuptool.runImportStepFromProfile(PROFILE_NAME, step, run_dependencies=False)

    # Remove useless alias_search_limit option in property sheet
    to_delete = 'alias_search_limit'
    propsheet = getPortal().portal_properties.collage_properties
    if propsheet.hasProperty(to_delete):
        propsheet._delProperty(to_delete)

    # Adding new properties
    to_add = [
        # Name, value, type
        ('ref_browser_empty', False, 'boolean'),
        ('ref_browser_types', False, 'boolean'),
        ('batch_size', 3, 'int')
        ]
    for name, value, ptype in to_add:
        if propsheet.hasProperty(name):
            continue
        propsheet.manage_addProperty(name, value, ptype)
    return




# -*- coding: utf-8 -*-
# $Id: viewlet.py 230152 2011-01-11 00:31:56Z dunlapm $

from ZODB.POSException import ConflictError

from OFS.CopySupport import _cb_decode
from OFS import Moniker

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Collage.interfaces import IDynamicViewManager
from Products.Collage.utilities import getCollageSiteOptions
from Products.Collage.utilities import CollageMessageFactory as _

from zope.component import getMultiAdapter, queryUtility

from plone.i18n.normalizer.interfaces import IIDNormalizer

class SimpleContentMenuViewlet(object):
    def isAlias(self):
        return getattr(self.__parent__, '__alias__', None) is not None

    def portal_url(self):
        return getToolByName(self.context, 'portal_url')()

    def getImmediateObject(self):
        """Returns the immediate object added to the Collage
        instead of a possible alias target object."""
        alias = getattr(self.__parent__, '__alias__', None)
        if alias:
            return alias.aq_inner

        return self.context.aq_inner

class LayoutViewlet(SimpleContentMenuViewlet):
    def getLayouts(self):
        context = self.context

        # handle aliased objects
        alias = getattr(self.__parent__, '__alias__', None)
        if alias: context = alias

        manager = IDynamicViewManager(context)

        # lookup active layout
        active = manager.getLayout()

        if not active:
            active, title = manager.getDefaultLayout()

        # compile list of registered layouts
        layouts = manager.getLayouts()

        # filter out fallback view
        layouts = filter(lambda (name, title): name != u'fallback', layouts)

        # make sure the active layout (which may not be available) is
        # included
        if active not in [name for name, title in layouts]:
            layouts.append(
                (active, _(u"Missing: $name", mapping={'name': active})))

        return [{
            'id': name,
            'name': title,
            'active': name == active} for (name, title) in layouts]

class SkinViewlet(SimpleContentMenuViewlet):
    def getSkins(self):
        context = self.context

        # handle aliased objects
        alias = getattr(self.__parent__, '__alias__', None)
        if alias: context = alias

        manager = IDynamicViewManager(context)

        # lookup active skin
        active = manager.getSkin()

        if not active:
            active = 'default'

        # compile list of registered skins for the layout
        skins = manager.getSkins(self.request)

        return [{'id': name,
                 'name': title,
                 'active': name == active} for (name, title) in skins]


class InsertNewItemViewlet(object):
    def normalizeString(self):
        return getToolByName(self.context, 'plone_utils').normalizeString

    def getAddableTypes(self):
        collage_options = getCollageSiteOptions()
        plone_view = self.context.restrictedTraverse('@@plone')
        container = plone_view.getCurrentFolder()
        idnormalizer = queryUtility(IIDNormalizer)
        
        #BBB Plone 4
        try:
            factories_view = getMultiAdapter((self.context, self.request),
                                             name='folder_factories')
            allowed_types = [i for i in factories_view.addable_types() if
                             collage_options.enabledType(i['id'])]
            
            return allowed_types
        except:
            portal_url = getToolByName(self.context, 'portal_url')()
            results = []
            for t in container.getAllowedTypes():
                if collage_options.enabledType(t.getId()):
                    cssId = idnormalizer.normalize(t.getId())
                    cssClass = 'contenttype-%s' % cssId
                    results.append({'title': t.Title(),
                                    'description': t.Description(),
                                    'icon': '%s/%s' % (portal_url, t.getIcon()),
                                    'extra': {'id' : cssId, 'separator' : None,
                                              'class' : cssClass},
                                    'id': t.getId()})

class SplitColumnViewlet(object):
    pass

class IconViewlet(SimpleContentMenuViewlet):
    def getIcon(self):
        tt = getToolByName(self.context, 'portal_types')
        obj_typeinfo = tt.getTypeInfo(self.context.portal_type)

        return obj_typeinfo.getIcon()

class ActionsViewlet(SimpleContentMenuViewlet):

    def getViewActions(self):
        plone_view = getMultiAdapter((self.context,self.request), name="plone")
        try:
            prepareObjectTabs = plone_view.prepareObjectTabs
        except AttributeError:
            provider = getMultiAdapter(
                (self.context, self.request, plone_view),
                name="plone.contentviews")
            viewlet = provider.__getitem__("plone.contentviews")
            prepareObjectTabs = viewlet.prepareObjectTabs
        return prepareObjectTabs()

class CopyViewlet(SimpleContentMenuViewlet):
    pass

class PasteViewlet(SimpleContentMenuViewlet):
    @property
    def clipboard_data_valid(self):
        cb_dataValid = getattr(self.context, 'cb_dataValid', None)

        if callable(cb_dataValid):
            return cb_dataValid()

    def _get_clipboard_item(self):
        cp = self.request.get('__cp')
        op, mdatas = _cb_decode(cp) # see OFS/CopySupport.py

        # just get the first item on the clipboard
        mdata = mdatas[0]
        m = Moniker.loadMoniker(mdata)

        app = self.context.getPhysicalRoot()
        try:
            ob = m.bind(app)
        except (ConflictError, AttributeError, KeyError, TypeError):
            return None

        return ob

    def render(self):
        """Only render if the clipboard contains an object that can
        be added to this container."""

        if self.clipboard_data_valid:
            # verify that we are allowed to paste this item here
            item = self._get_clipboard_item()

            if item:
                portal_type = item.portal_type
                allowed_types = self.context.allowedContentTypes()
                if portal_type in (t.getId() for t in allowed_types):
                    return self.index()

        return u''

    def clipboard_item_title(self):
        title = self._get_clipboard_item().title_or_id()
        return u'Paste "%s"' % safe_unicode(title)



# -*- coding: utf-8 -*-
# $Id: helper.py 106369 2009-12-10 12:03:19Z mborch $

from zope.interface import Interface

from Acquisition import aq_inner, aq_parent

from Products.Collage.interfaces import ICollage

class ICollageHelper(Interface):
    def isCollageContent():
        """True if the content item is in a Collage"""

    def getCollageObject():
        """Search object tree for a Collage object or None."""

    def getCollageObjectURL():
        """Search object tree for a Collage-object and return URL."""


class CollageHelper(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def isCollageContent(self, parent=None):
        return self.getCollageObject() is not None

    def getCollageObject(self, parent=None):
        if parent is None:
            parent = self.context

        while parent is not None:
            parent = aq_parent(aq_inner(parent))

            if ICollage.providedBy(parent):
                return parent

    def getCollageObjectURL(self, parent=None):
        collage = self.getCollageObject(parent)
        if collage:
            return collage.absolute_url()
        raise ValueError("Collage object not found.")



# -*- coding: utf-8 -*-
# $Id: existingitems.py 235454 2011-03-03 12:57:29Z mborch $

from zope.component import getMultiAdapter
from zope.component import getUtility, queryUtility

try:
    from zope.schema.interfaces import IVocabularyFactory
except ImportError:
    from zope.app.schema.vocabulary import IVocabularyFactory

from plone.memoize.view import memoize_contextless
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.i18n.normalizer.interfaces import IIDNormalizer

from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getSiteEncoding
from Products.CMFPlone import PloneMessageFactory as p_
from Products.CMFPlone import utils
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs

try:
    from Products.CMFPlone import Batch
except ImportError:
    from Products.CMFPlone.PloneBatch import Batch

from Products.Collage.browser.utils import escape_to_entities
from Products.Collage.utilities import getCollageSiteOptions
from Products.ZCTextIndex.ParseTree import ParseError

try:
    from plone.app.layout.navigation.interfaces import INavigationRoot
except ImportError:
    from Products.CMFPlone.interfaces import IPloneSiteRoot as INavigationRoot


from urllib import unquote


class ExistingItemsView(BrowserView):
    limit = 10

    def __call__(self):
        data = self._data()
        content = self.index(**data)

        # IE6 encoding bug workaround (IE6 sucks but...)
        if self.request.get('USER_AGENT', '').find('MSIE 6.0') > 0:
            # response must be latin-1
            self.request.RESPONSE.setHeader("Content-Type", "text/html; charset=ISO-8859-1")
            encoding = getSiteEncoding(self.context.context)
            if not isinstance(content, unicode):
                content = content.decode(encoding)

            # convert special characters to HTML entities since we're
            # recoding to latin-1
            content = escape_to_entities(content).encode('latin-1')

        return content

    @property
    def catalog(self):
        return getMultiAdapter((self.context, self.request),
                               name=u'plone_tools').catalog()

    @property
    def navigation_root(self):
        context = self.context.aq_inner
        while context is not None:
            if INavigationRoot.providedBy(context):
                break

            context = context.aq_parent

        return context

    def portal_url(self):
        return getMultiAdapter((self.context, self.request),
                               name=u'plone_portal_state').portal_url()

    @property
    def typefilter(self):
        options = getCollageSiteOptions()
        if not options.ref_browser_types:
            return list()
        ret = [{'title': '', 'value': '', 'selected': False}]
        factory = getUtility(IVocabularyFactory,
                         name=u'collage.vocabularies.CollageUserFriendlyTypes')
        for term in factory(self.context):
            if options.use_whitelist:
                if not term.value in options.alias_whitelist:
                    continue
            selected = self.request.form.get('portal_type') == term.value \
                and True or False
            ret.append({
                'title': term.title,
                'value': term.value,
                'selected': selected,
            })
        return ret
    
    def breadcrumbs(self):
        path = self.request.get('path')
        portal = self.navigation_root
        if path is None:
            context = portal
        else:
            context = portal.restrictedTraverse(unquote(path))
        crumbs = []
        context = aq_inner(context)
        while context is not None:
            if not IHideFromBreadcrumbs.providedBy(context):
                crumbs.append({
                    'path': "/".join(context.getPhysicalPath()),
                    'url': context.absolute_url(),
                    'title': context.title_or_id()
                    })
            if INavigationRoot.providedBy(context):
                break
            context = utils.parent(context)
        crumbs.reverse()
        return crumbs

    @memoize_contextless
    def listEnabledTypes(self):
        """Enabled types in a Collage as list of dicts.
        """
        actual_portal_type = self.request.get('portal_type', None)
        collage_options = getCollageSiteOptions()
        ttool = getToolByName(self.context, 'portal_types', None)
        if ttool is None:
            return None
        return [{'id': pt.getId(),
                 'title': p_(pt.Title()),
                 'selected': pt.getId() == actual_portal_type and 'selected' or None}
                for pt in ttool.listTypeInfo()
                if collage_options.enabledAlias(pt.getId())]
    
    @property
    def readitems(self):
        options = getCollageSiteOptions()
        if options.ref_browser_empty \
          and self.request.form.get('portal_type', '') == '' \
          and self.request.form.get('path', '') == '' \
          and self.request.form.get('b_start', None) is None:
            return False
        return True

    def _data(self):
        portal_types = self.request.get('portal_type', None)
        if not portal_types:
            portal_types = [pt['id'] for pt in self.listEnabledTypes()]
        query_path = self.request.get('path') or \
                     "/".join(self.navigation_root.getPhysicalPath())
        query_path = unquote(query_path)
        b_start = self.request.get('b_start', 0)
        query_text = unquote(self.request.get('SearchableText', ''))
        if query_text:
            depth = 10
        else:
            depth = 1
        if not self.readitems:
            results = []
        else:
            try:
                results = self.catalog(
                    Language='all',
                    SearchableText=query_text,
                    portal_type=portal_types,
                    path={"query": query_path, 'depth': depth},
                    sort_on='sortable_title',
                    sort_order='ascending')
                # alternative: sort_order='reverse', sort_on='modified'
            except ParseError:
                results = []

        # setup description cropping
        cropText = getMultiAdapter(
            (self.context, self.request), name=u'plone').cropText
        props = getMultiAdapter((self.context, self.request),
                                name=u'plone_tools').properties()
        site_properties = props.site_properties
        desc_length = getattr(site_properties, 'search_results_description_length', 25)
        desc_ellipsis = getattr(site_properties, 'ellipsis', '...')

        items = []
        batch = Batch(results, self.limit, int(b_start), orphan=1)
        idnormalizer = queryUtility(IIDNormalizer)
        for result in batch:
            cssType = idnormalizer.normalize(result.portal_type)
            items.append({
                'UID': result.UID,
                'icon' : result.getIcon,
                'title': result.Title,
                'description': cropText(result.Description, desc_length, desc_ellipsis),
                'type': result.Type,
                'folderish': result.is_folderish,
                'target_url': result.getURL(),
                'path': result.getPath(),
                'link_css_class': 'state-%s' % result.review_state,
                'cssType': 'contenttype-%s' % cssType,
                })

        return {
            'items': items,
            'batch': batch,
            'query': bool(query_text),
            'query_text': query_text,
            'path': query_path,
            }



from Products.Five.browser import BrowserView

from Products.Collage.interfaces import IDynamicViewManager
from Products.Collage.interfaces import ICollageAlias
from Products.Collage.viewmanager import mark_request

from zope.interface import Interface, directlyProvides

from zope.component import getMultiAdapter

from Acquisition import aq_inner

class IKSSHelper(Interface):
    def getUniqueIdentifier():
        pass

    def getKssView():
        pass

    def getKssClasses(field_name):
        pass

class KSSHelper(BrowserView):
    """To better support various Plone environments we implement
    this view to help generate the right inline-editing bindings."""

    def getUniqueIdentifier(self):
        return self.context.UID()

    def getKssView(self):
        return self.context.restrictedTraverse('@@kss_field_decorator_view')

    def getKssClasses(self, field_name):
        kss = self.getKssView()

        # choose appropriate kss class generator depending on rendering mode
        if self.request.get('URL').endswith('/replaceField'):
            f = kss.getKssClasses
        else:
            f = kss.getKssClassesInlineEditable

        editing_classes = f(field_name,
                            templateId='kss_collage_macro_proxy',
                            macro=field_name,
                            target="%s-%s" % (field_name, self.getUniqueIdentifier()))
        uid_class = kss.getKssUIDClass()
        return " ".join((editing_classes, uid_class))

class CollageMacrosView(BrowserView):
    """This helper view looks up the current view for the context-object
    and returns it without rendering it."""

    @property
    def __call__(self):
        context = aq_inner(self.context)

        manager = IDynamicViewManager(context)
        layout = manager.getLayout()

        if not layout:
            layout, title = manager.getDefaultLayout()

        if ICollageAlias.providedBy(context):
            context = context.get_target()

            # if not set, revert to self.context
            if not context: context = self.context

        # transmute request interfaces
        ifaces = mark_request(self.context, self.request)

        view = getMultiAdapter((context, self.request), name=layout)

        # restore interfaces
        directlyProvides(self.request, ifaces)

        return view.index

class DummyFieldsView(BrowserView):
    def getKssUIDClass(self):
        return ''

    def getKssClassesInlineEditable(self, *args, **kwargs):
        return ''

    def getKssClasses(self, *args, **kwargs):
        return ''



# $Id$
"""Testing Collage utilities"""

from Products.Collage.tests.base import CollageTestCase
from Products.Collage.tests import utils as ctc_utils
from Products.Collage import utilities as cu

class UtilitiesTestCase(CollageTestCase):
    def testGenerateNewId(self):
        self.loginAsPortalOwner()
        self.portal.REQUEST.environ['REQUEST_METHOD'] = 'POST'
        foo_collage = ctc_utils.addCollage(self.portal, 'foo', 'Foo')

        new_id = cu.generateNewId(foo_collage)
        self.failUnlessEqual(new_id, '1')

        foo_collage.restrictedTraverse('insert-row').insertRow()
        new_id = cu.generateNewId(foo_collage)
        self.failUnlessEqual(new_id, '2')

        foo_collage.restrictedTraverse('insert-row').insertRow()
        self.failUnlessEqual(foo_collage.objectIds()[-1], '2')

        foo_collage._delObject(foo_collage.objectIds()[0])
        new_id = cu.generateNewId(foo_collage)
        self.failUnlessEqual(new_id, '1')
        return

    def testIsTranslatable(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory('Document', 'doc', title="Doc")
        doc = getattr(self.portal, 'doc')
        if cu.HAS_LINGUAPLONE:
            self.failUnless(cu.isTranslatable(doc))
        else:
            self.failIf(cu.isTranslatable(doc))
        return

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(UtilitiesTestCase))
    return suite



import htmlentitydefs

"""
Unicode to HTML entity-converter (neccesary for IE6)
"""

codepoint2entity = {}
safe_characters = ['<', '>', '"', '&']
for c in htmlentitydefs.codepoint2name:
    if c not in map(ord, safe_characters): # skip "safe" characters
        codepoint2entity[c] = '&%s;' % unicode(htmlentitydefs.codepoint2name[c])

def escape_to_entities(string):
    ustr = string.translate(codepoint2entity)
    result = []
    for s in ustr:
        if ord(s) > 0x7f:
            s = '&#%d;' % ord(s)
        result.append(s)

    return "".join(result)



# -*- coding: utf-8 -*-
# $Id: __init__.py 76587 2008-11-29 16:33:21Z glenfant $
"""Collage content types"""

from Products.Collage.content._collage import Collage
from Products.Collage.content._row import CollageRow
from Products.Collage.content._column import CollageColumn
from Products.Collage.content._alias import CollageAlias



# $Id: common.py 72630 2008-09-28 15:17:35Z thomasw $

from Products.Archetypes import atapi
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Collage.interfaces import ICollageColumn

class LayoutContainer(object):
    """
    Container that provides aggregate search and display
    functionality.
    """

    def canSetDefaultPage(self):
        """Based on RichDocument/content/richdocument.py
        This method, from ISelectableBrowserDefault, is used to check whether
        the 'Choose content item to use as deafult view' option will be
        presented. This makes sense for folders, but not for RichDocument, so
        always disallow.
        """
        return False

    def aggregateSearchableText(self):
        """Append references' searchable fields."""

        data = super(LayoutContainer, self).SearchableText()

        # Must we add subcontents texts?
        if ICollageColumn.providedBy(self):
            helper = self.restrictedTraverse('@@collage_helper')
            collage = helper.getCollageObject()
            if not collage.mustIndexSubobjects():
                return data

        data = [data]
        for child in self.contentValues():
            data.append(child.SearchableText())

        data = ' '.join(data)
        return data

from Products.ATContentTypes.content.schemata import ATContentTypeSchema
CommonCollageSchema = atapi.Schema((
    ATContentTypeSchema['excludeFromNav'].copy(),
    ATContentTypeSchema['relatedItems'].copy(),
))


"""
Misc utilities for testing
"""
from zope.interface import implements

from zope.annotation import IAttributeAnnotatable
from zope.publisher.browser import TestRequest as ZopeTestRequest
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.interfaces.browser import IBrowserRequest

class TestRequest(ZopeTestRequest):
    implements(IHTTPRequest, IAttributeAnnotatable, IBrowserRequest)

    def set(self, attribute, value):
        self._environ[attribute] = value

def addCollage(container, oid, title):
    """Adding an empty Collage obj"""
    container.invokeFactory('Collage', oid, title=title)
    collage = getattr(container, oid)
    return collage



# $Id: renderer.py 108068 2010-01-05 14:18:34Z mborch $

from AccessControl import getSecurityManager, Unauthorized
from Globals import DTMLFile

from zope.interface import directlyProvides
from zope.component import queryMultiAdapter
from zope.component import getMultiAdapter
from zope.component import ComponentLookupError

from Products.Five.browser import BrowserView

from Products.Collage.interfaces import IDynamicViewManager
from Products.Collage.interfaces import ICollageAlias
from Products.Collage.utilities import isTranslatable
from Products.Collage.viewmanager import mark_request

class SimpleContainerRenderer(BrowserView):

    def getItems(self, contents=None):
        """Items are a views to render.

        @param contents: If given fetch the folderListingFolderContents of
                         context.
        @return: a list of views ready to render.
        """
        # needed to circumvent bug :-(
        self.request.debug = False

        # transmute request interfaces
        ifaces = mark_request(self.context, self.request)

        views = []
        if contents is None:
            contents = self.context.folderlistingFolderContents()
        for context in contents:
            if context is None:
                continue
            target = context
            manager = IDynamicViewManager(context)
            layout = manager.getLayout()

            if not layout:
                layout, title = manager.getDefaultLayout()

            if ICollageAlias.providedBy(context):
                target = context.get_target()

                # if not set, revert to context
                if target is None:
                    target = context

                # verify that target is accessible
                try:
                    getSecurityManager().validate(self, self, target.getId(), target)
                except Unauthorized:
                    continue

            # Filter out translation duplicates:
            # If a non-alias object is translatable, check if its language
            # is set to the currently selected language or to neutral,
            # or if it is the canonical version
            elif isTranslatable(target):
                language = self.request.get('LANGUAGE','')
                if target.Language() not in (language, ''):
                    # Discard the object, if it is not the canonical version
                    # or a translation is available in the requested language.
                    if not target.isCanonical() or target.getTranslation(language) in contents:
                        continue
                # If the target is a translation, get the layout defined on the canonical
                # object, unless a layout has already been defined on the translation.
                # Fallback to default layout.
                if not target.isCanonical():
                    canmanager = IDynamicViewManager(target.getCanonical())
                    layout = manager.getLayout() or canmanager.getLayout() or layout

            # don't assume that a layout is always available; note
            # that we can't use ``queryMultiAdapter`` because even
            # this lookup might fail hard
            try:
                view = getMultiAdapter((target, self.request), name=layout)
            except ComponentLookupError:
                view = getMultiAdapter(
                    (target, self.request), name='error_collage-view-not-found')
                view.notfoundlayoutname = layout

            # store reference to alias if applicable
            if ICollageAlias.providedBy(context):
                view.__alias__ = context

            views.append(view)

        # restore interfaces
        directlyProvides(self.request, ifaces)
        return views

class CollageStylesheet(BrowserView):
    """Renders the collage standard stylesheet
    """
    template = DTMLFile('templates/collage.css', globals())

    def __call__(self, *args, **kwargs):
        """Renders the standard collage stylesheet.
        Note that we do not change HTTP headers since we are supposed to be
        published through the CSS registry
        """
        template = self.template.__of__(self.context)
        return template(context=self.context, request=self.request)



from zope.interface import (
    Interface, implements, providedBy, directlyProvidedBy,
    directlyProvides)

from zope.component import (
    getSiteManager, getMultiAdapter, getUtilitiesFor, queryUtility)
from zope.component import ComponentLookupError

from zope.annotation.interfaces import IAnnotations
from persistent.dict import PersistentDict

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName

from interfaces import IDynamicViewManager
from interfaces import ICollageAlias
from interfaces import ICollageBrowserLayer 
from interfaces import ICollageBrowserLayerType

ANNOTATIONS_KEY = u'Collage'

class DynamicViewManager(object):
    implements(IDynamicViewManager)

    def __init__(self, context):
        self.context = context

    def getStorage(self):
        annotations = IAnnotations(self.context)
        return annotations.setdefault(ANNOTATIONS_KEY, PersistentDict())

    def getLayout(self):
        storage = self.getStorage()
        return storage.get('layout', None)

    def setLayout(self, layout):
        storage = self.getStorage()
        storage['layout'] = layout

    def getDefaultLayout(self):
        layouts = self.getLayouts()

        if layouts:
            # look for a standard view (by naming convention)
            for name, title in layouts:
                if name == u'standard':
                    return (name, title)

            # otherwise return first view factory
            return layouts[0]

        raise ValueError

    def getLayouts(self):
        context = self.context
        request = context.REQUEST

        if ICollageAlias.providedBy(self.context):
            target = self.context.get_target()
            if target is not None:
                context = target

        ifaces = mark_request(context, request)

        sm = getSiteManager()
        layouts = sm.adapters.lookupAll(
            required=(providedBy(context), providedBy(request)),
            provided=Interface
            )

        directlyProvides(request, *ifaces)
        layouts = [(name, getattr(layout, 'title', name))
                   for (name, layout) in layouts
                   if isinstance(layout, type) and
                      issubclass(layout, BrowserView) and
                      not getattr(layout, 'hide', False)]
        layouts.sort(lambda a, b: cmp(a[1], b[1]))
        return layouts

    def getSkin(self):
        storage = self.getStorage()
        return storage.get('skin', None)


    def setSkin(self, skin):
        storage = self.getStorage()
        storage['skin'] = skin


    def getSkins(self, request=None):

        layout = self.getLayout()
        skins = []

        if layout and request:
            request.debug = False

            ifaces = mark_request(self.context, request)

            target = self.context
            if ICollageAlias.providedBy(target):
                target = target.get_target()
                if not target:
                    target = self.context

            try:
                view = getMultiAdapter((target, request), name=layout)
                skinInterfaces = getattr(view, 'skinInterfaces', ())
            except ComponentLookupError:
                skinInterfaces = ()

            for si in skinInterfaces:
                for name, utility in getUtilitiesFor(si):
                    skins.append((name, utility.title))

            # restore interfaces
            directlyProvides(request, ifaces)

        skins.sort(lambda x, y: cmp(x[0], y[0]))
        return skins


def mark_request(context, request):
    """ Marks the request with general and theme-specific Collage browser layers.

        Returns the initial set of request marker interfaces so that you can restore
        them using directlyProvides(request, ifaces).
    """
    initial_ifaces = directlyProvidedBy(request)

    # general Collage layer
    directlyProvides(request, ICollageBrowserLayer)

    # theme-specific Collage layer
    portal_skins = getToolByName(context, 'portal_skins', None)
    if portal_skins is not None:
        skin_name = portal_skins.getCurrentSkinName()
        layer_iface = queryUtility(ICollageBrowserLayerType, name=skin_name)
        if layer_iface is not None:
            directlyProvides(request, layer_iface, ICollageBrowserLayer)

    return initial_ifaces



"""
Common testing resources
"""
from Testing import ZopeTestCase

ZopeTestCase.installProduct('Collage')

from Products.PloneTestCase import PloneTestCase

PloneTestCase.setupPloneSite(products=('Collage',))


class CollageTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()

class CollageFunctionalTestCase(PloneTestCase.FunctionalTestCase):
    
    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.FunctionalTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()



# -*- coding: utf-8 -*-
# $Id: collage.py 106309 2009-12-09 16:11:57Z mborch $

from zope.interface import alsoProvides
from zope.interface import noLongerProvides

from Acquisition import aq_base

from Products.Five.browser import BrowserView
from Products.CMFPlone.interfaces import INonStructuralFolder
from Products.Collage.interfaces import ICollageEditLayer

class CollageView(BrowserView):
    def isStructuralFolder(self, instance):
        context = instance
        folderish = bool(getattr(aq_base(context), 'isPrincipiaFolderish',
                                 False))
        if not folderish:
            return False
        elif INonStructuralFolder.providedBy(context):
            return False
        else:
            return folderish

class CollageComposeView(CollageView):
    def __call__(self):
        alsoProvides(self.request, ICollageEditLayer)
        result = super(CollageComposeView, self).__call__()
        noLongerProvides(self.request, ICollageEditLayer)
        return result




# $Id: _column.py 70990 2008-09-01 13:19:21Z glenfant $

from AccessControl import ClassSecurityInfo

from Products.Archetypes import atapi
from Products.Collage.content.common import LayoutContainer,CommonCollageSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.Collage.interfaces import ICollageColumn

from zope.interface import implements

# CMFDynamicViewFTI imports
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.Collage.utilities import CollageMessageFactory as _

CollageColumnSchema = atapi.BaseContent.schema.copy() + atapi.Schema((
    atapi.StringField(
        name='title',
        accessor='Title',
        required=False,
        searchable=True,
        widget=atapi.StringWidget(
            label=_(u'label_optional_column_title', default=u"Title"),
            description=_(u'help_optional_column_title',
                          default=u"You may optionally supply a title for this column."),
        )
    ),
))

CollageColumnSchema = CollageColumnSchema + CommonCollageSchema.copy()

# move description to main edit page
CollageColumnSchema['description'].schemata = 'default'

# never show row in navigation, also when its selected
CollageColumnSchema['excludeFromNav'].default = True

# support show in navigation feature and at marshalling
finalizeATCTSchema(CollageColumnSchema, folderish=True, moveDiscussion=False)

class CollageColumn(BrowserDefaultMixin, LayoutContainer, atapi.OrderedBaseFolder):

    # FIXME: Do we have to honour Zope 2 style interface?
    __implements__ = (getattr(atapi.OrderedBaseFolder,'__implements__',()), \
                      getattr(BrowserDefaultMixin,'__implements__',()))

    schema = CollageColumnSchema

    _at_rename_after_creation = True

    implements(ICollageColumn)

    security = ClassSecurityInfo()

    def SearchableText(self):
        return self.aggregateSearchableText()

    def indexObject(self):
        pass

    def reindexObject(self,idxs=[] ):
        pass

    def unindexObject(self):
        pass

atapi.registerType(CollageColumn, 'Collage')



# $Id: _collage.py 106308 2009-12-09 15:54:24Z mborch $

from AccessControl import ClassSecurityInfo

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi

from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import ATCTMixin
from Products.Collage.content.common import LayoutContainer, CommonCollageSchema

from Products.CMFPlone.interfaces import INonStructuralFolder

from Products.Collage.interfaces import ICollage

from zope.interface import implements

from Products.Collage.utilities import CollageMessageFactory as _

CollageSchema = atapi.BaseContent.schema.copy() + atapi.Schema((
    atapi.StringField(
        name='title',
        searchable=True,
        widget=atapi.StringWidget(
            label='Title',
            label_msgid='label_title',
            i18n_domain='plone',
        )
    ),

    atapi.TextField(
        name='description',
        searchable=True,
        widget=atapi.TextAreaWidget(
            label='Description',
            label_msgid='label_description',
            i18n_domain='plone',
        )
    ),
    atapi.BooleanField('show_title',        
        accessor='getShowTitle',
        widget=atapi.BooleanWidget(
            label=_(u'label_show_title', default=u"Show title"),
            description=_(u'help_show_title', default=u"Show title in page composition.")),
        default=1,
        languageIndependent=True,
        schemata="settings"),

    atapi.BooleanField('show_description',
        accessor='getShowDescription',
        widget=atapi.BooleanWidget(
            label=_(u'label_show_description', default='Show description'),
            description=_(u'help_show_description', default=u"Show description in page composition.")),
        default=1,
        languageIndependent=True,
        schemata="settings"),

    atapi.BooleanField('index_subobjects',
        accessor='mustIndexSubobjects',
        default=False,
        languageIndependent=True,
        schemata="settings",
        widget=atapi.BooleanWidget(
            label=_(u'label_index_subobjects', default=u"Add collage contents in searchable text?"),
            description=_(u'help_index_subobjects',
                 default=u"Show this collage in results when searching for terms "
                          u"appearing in a contained item. "
                          u"Note: Checking this option may slow down the system "
                          u"while editing the collage."))
            )


))

CollageSchema = CollageSchema + CommonCollageSchema.copy()

# move description to main edit page
CollageSchema['description'].schemata = 'default'

# support show in navigation feature and at marshalling
# speciel case set folderish to False since we want related items to be used
finalizeATCTSchema(CollageSchema, folderish=False, moveDiscussion=False)

class Collage(LayoutContainer, ATCTMixin, atapi.OrderedBaseFolder):

    # FIXME: Do we always need Zope 2 style interfaces ?
    __implements__ = (getattr(atapi.OrderedBaseFolder,'__implements__',()),
                      getattr(ATCTMixin, '__implements__',()))

    schema = CollageSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    implements(ICollage, INonStructuralFolder)

    def SearchableText(self):
        return self.aggregateSearchableText()

atapi.registerType(Collage, 'Collage')



# -*- coding: utf-8 -*-
# $Id: config.py 83112 2009-03-25 09:50:10Z jensens $
"""Collage config constants"""

PROJECTNAME = "Collage"
I18N_DOMAIN = PROJECTNAME.lower()
PROPERTYSHEETNAME = 'collage_properties'
COLLAGE_TYPES = ('Collage', 'CollageRow', 'CollageColumn', 'CollageAlias')

from Products.CMFCore.permissions import setDefaultRoles

DEFAULT_ADD_CONTENT_PERMISSION = "Add Collage content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner', 'Contributor'))

import os
PACKAGE_HOME = os.path.dirname(os.path.abspath(__file__))
del os



# -*- coding: utf-8 -*-
# $Id: actions.py 110283 2010-02-04 10:20:38Z mborch $

from zope import event
from zope.lifecycleevent import ObjectModifiedEvent

from Products.Five.browser import BrowserView
from Products.CMFPlone import utils as cmfutils
from Products.Collage.utilities import CollageMessageFactory as _
from Products.Collage.utilities import generateNewId, findFirstAvailableInteger
from Products.Collage.interfaces import IDynamicViewManager

from plone.protect.postonly import check

def protect(func):
    def wrapper(self):
        check(self.request)
        return func(self)
    wrapper.__name__ = func.__name__
    return wrapper

class ActionsView(BrowserView):
    @protect
    def setDynamicView(self):
        layout = self.request['layout']

        manager = IDynamicViewManager(self.context)
        manager.setLayout(layout)

        self.context.plone_utils.addPortalMessage(_(u'View changed.'))
        self.request.response.redirect(self.context.REQUEST['HTTP_REFERER'])

    @protect
    def setDynamicSkin(self):
        skin = self.request['skin']

        manager = IDynamicViewManager(self.context)
        manager.setSkin(skin)

        self.context.plone_utils.addPortalMessage(_(u'Skin changed.'))
        self.request.response.redirect(self.context.REQUEST['HTTP_REFERER'])

    @protect
    def reorderObjects(self):
        object_id = self.request['id']
        position = self.request['position']

        if position.lower()=='up':
            self.context.moveObjectsUp(object_id)

        if position.lower()=='down':
            self.context.moveObjectsDown(object_id)

        if position.lower()=='top':
            self.context.moveObjectsToTop(object_id)

        if position.lower()=='bottom':
            self.context.moveObjectsToBottom(object_id)

        # order folder by field
        # id in this case is the field
        if position.lower()=='ordered':
            self.context.orderObjects(object_id)

        cmfutils.getToolByName(self.context, 'plone_utils').reindexOnReorder(self.context)
        return 1

    @protect
    def insertRow(self):
        # create row
        desired_id = generateNewId(self.context)
        row_id = self.context.invokeFactory(id=desired_id, type_name='CollageRow')
        row = getattr(self.context, row_id, None)
        row.setTitle('')

        # create column
        desired_id = generateNewId(row)
        col_id = row.invokeFactory(id=desired_id, type_name='CollageColumn')
        col = getattr(row, col_id, None)
        col.setTitle('')

        self.context.plone_utils.addPortalMessage(_(u'msg_row_added', default=u"Row was added."))
        self.request.response.redirect(self.context.REQUEST['HTTP_REFERER'])

    @protect
    def insertColumn(self):
        container = self.context
        desired_id = generateNewId(container)

        container.invokeFactory(id=desired_id, type_name='CollageColumn')

        self.context.plone_utils.addPortalMessage(_(u'msg_column_inserted', default="Column inserted."))
        self.request.response.redirect(self.context.REQUEST['HTTP_REFERER'])

    @protect
    def insertAlias(self):
        uid_catalog = cmfutils.getToolByName(self.context,
                                             'uid_catalog')

        uid = self.request.get('uid')
        container = self.context

        # check that target object exists
        brains = uid_catalog(UID=uid)
        if brains:
            # find first available id for the alias object
            ids = [i[6:] for i in container.objectIds() if i.startswith('alias-')]
            alias_id = 'alias-%s' % findFirstAvailableInteger(ids)

            # create new alias
            container.invokeFactory('CollageAlias', id=alias_id)
            alias = container[alias_id]

            # set target
            alias.set_target(uid)
            event.notify(ObjectModifiedEvent(alias))
            msg = _(u'msg_alias_inserted', default=u"Alias inserted.")
        else:
            msg = _(u'msg_target_object_not_found', default=u"Target object not found.")

        referer = self.request.get('HTTP_REFERER', self.context.absolute_url())
        self.context.plone_utils.addPortalMessage(msg)
        return self.request.RESPONSE.redirect(referer)

    @protect
    def deleteObject(self):
        parent = self.context.aq_inner.aq_parent
        parent.manage_delObjects(self.context.getId())
        fti = self.context.portal_types[self.context.portal_type]
        title = self.context.translate(fti.title, domain='plone')
        message = _(u'${title} deleted.', mapping={u'title' : title})
        self.context.plone_utils.addPortalMessage(message)
        self.request.response.redirect(self.context.REQUEST['HTTP_REFERER'])



# $Id: _alias.py 101037 2009-11-02 00:54:05Z glenfant $

from zope.interface import implements
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.Archetypes import atapi
from Products.Archetypes.ReferenceEngine import Reference
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from Products.ATContentTypes.content.base import ATCTContent

from Products.Collage.interfaces import ICollageAlias, IDynamicViewManager
from Products.Collage.content.common import LayoutContainer
from Products.Collage.utilities import CollageMessageFactory as _
from Products.Collage.utilities import isTranslatable

class CollageAliasReference(Reference):
    """We need our own reference class to have a custom target deletion hook
    that resets the alias layout.
    """
    def delHook(self, tool, sourceObject=None, targetObject=None):
        """Override standard delHook
        """
        manager = IDynamicViewManager(sourceObject)
        manager.setLayout(None)
        return


CollageAliasSchema = ATCTContent.schema.copy() + atapi.Schema((
    atapi.ReferenceField(
        name='target',
        mutator='set_target',
        accessor='get_target',
        referenceClass = CollageAliasReference,
        relationship='Collage_aliasedItem',
        multiValued = 0,
        allowed_types = (),
        widget=ReferenceBrowserWidget(
            label=_(u'label_alias_target', default="Selected target object"),
            startup_directory='/',
        ),
        keepReferencesOnCopy=True,
    ),
))

# we don't require any fields to be filled out
CollageAliasSchema['title'].required = False

# never show in navigation, also when its selected
CollageAliasSchema['excludeFromNav'].default = True

CollageAliasSchema['relatedItems'].widget.visible = {'edit':'invisible', 'view':'invisible'}

class CollageAlias(BrowserDefaultMixin, LayoutContainer, ATCTContent):
    __implements__ = (getattr(atapi.OrderedBaseFolder,'__implements__',()), \
                      getattr(BrowserDefaultMixin,'__implements__',()))

    implements(ICollageAlias)

    meta_type = 'CollageAlias'

    schema = CollageAliasSchema
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    def indexObject(self):
        pass

    def reindexObject(self,idxs=[] ):
        pass

    def unindexObject(self):
        pass

    def get_target(self):
        res = self.getRefs(self.getField('target').relationship)
        if res:
            res = res[0]
        else:
            res = None
        if isTranslatable(res):
            lang = getToolByName(self, 'portal_languages').getPreferredLanguage()
            res = res.getTranslation(lang) or res
        return res

atapi.registerType(CollageAlias, 'Collage')



# -*- coding: utf-8 -*-
# $Id: __init__.py 76360 2008-11-23 22:25:42Z glenfant $

from Products.Archetypes.atapi import listTypes, process_types

GLOBALS = globals()

from config import DEFAULT_ADD_CONTENT_PERMISSION, PROJECTNAME

def initialize(context):
    from Products.Collage import content
    from Products.CMFCore import utils as cmfutils

    dummy = content # Keep pyflakes silent
    # initialize the content, including types and add permissions
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    cmfutils.ContentInit('%s Content' % PROJECTNAME,
                         content_types = content_types,
                         permission = DEFAULT_ADD_CONTENT_PERMISSION,
                         extra_constructors = constructors,
                         fti = ftis,
                         ).initialize(context)



# -*- coding: utf-8 -*-
# $Id: controlpanel.py 235454 2011-03-03 12:57:29Z mborch $
"""Collage site wide options"""

from zope.component import adapts
from zope.interface import implements
try:
    from zope.schema.interfaces import IVocabularyFactory
except ImportError:
    from zope.app.schema.vocabulary import IVocabularyFactory

from zope.schema.vocabulary import SimpleVocabulary
from zope.schema.vocabulary import SimpleTerm
from zope.formlib.form import FormFields
from plone.app.vocabularies.types import BAD_TYPES
from plone.app.controlpanel.form import ControlPanelForm
from plone.app.controlpanel.widgets import MultiCheckBoxThreeColumnWidget
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFDefault.formlib.schema import ProxyFieldProperty
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Collage.interfaces import ICollageSiteOptions
from Products.Collage.config import PROPERTYSHEETNAME
from Products.Collage.config import COLLAGE_TYPES
from Products.Collage.utilities import getPortal
from Products.Collage.utilities import CollageMessageFactory as _

class CollageSiteOptions(SchemaAdapterBase):

    implements(ICollageSiteOptions)
    adapts(IPloneSiteRoot)

    ref_browser_empty = \
        ProxyFieldProperty(ICollageSiteOptions['ref_browser_empty'])
    ref_browser_types = \
        ProxyFieldProperty(ICollageSiteOptions['ref_browser_types'])
    use_whitelist = ProxyFieldProperty(ICollageSiteOptions['use_whitelist'])
    types_whitelist = ProxyFieldProperty(ICollageSiteOptions['types_whitelist'])
    alias_whitelist = ProxyFieldProperty(ICollageSiteOptions['alias_whitelist'])
    batch_size = ProxyFieldProperty(ICollageSiteOptions['batch_size'])

    def __init__(self, context):
        super(CollageSiteOptions, self).__init__(context)
        properties = getPortal().portal_properties
        self.context = properties.restrictedTraverse(PROPERTYSHEETNAME)

    def enabledType(self, portal_type):
        if portal_type in COLLAGE_TYPES:
            return False
        if self.use_whitelist:
            return portal_type in self.types_whitelist
        return True

    def enabledAlias(self, portal_type):
        if portal_type in COLLAGE_TYPES:
            return False
        if self.use_whitelist:
            return portal_type in self.alias_whitelist
        return True

class CollageControlPanel(ControlPanelForm):

    form_fields = FormFields(ICollageSiteOptions)
    form_fields['types_whitelist'].custom_widget = MultiCheckBoxThreeColumnWidget
    form_fields['alias_whitelist'].custom_widget = MultiCheckBoxThreeColumnWidget

    label = form_name = _(u'title_collage_controlpanel',
                          default=u"Collage control panel")
    description = _(u'help_collage_controlpanel',
                    default=u"Site wide options for Collage")

class CollageUserFriendlyTypesVocabulary(object):
    """Vocabulary"""
    implements(IVocabularyFactory)

    def __call__(self, context):
        context = getattr(context, 'context', context)
        ttool = getToolByName(context, 'portal_types', None)
        if ttool is None:
            return None
        items = [(t, t, ttool[t].Title())
                  for t in ttool.listContentTypes()
                  if t not in BAD_TYPES + COLLAGE_TYPES]
        items = [SimpleTerm(*v) for v in items]
        return SimpleVocabulary(items)

CollageUserFriendlyTypesVocabularyFactory = CollageUserFriendlyTypesVocabulary()



"""
Running doctests
"""

"""Ploneboard functional doctests.  This module collects all *.txt
files in the tests directory and runs them. (stolen from Plone)
"""

import os, sys

import glob
import doctest
import unittest
from Globals import package_home
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.Collage.tests.base import CollageFunctionalTestCase

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def list_doctests():
    this_dir = os.path.dirname(os.path.abspath(__file__))
    filenames = [filename for filename in glob.glob(os.path.sep.join([this_dir, '*.txt']))]
    if not HAS_LINGUA_PLONE:
        filenames = [filename for filename in filenames
                     if not filename.endswith('multilingual_support.txt')]
    return filenames

def test_suite():
    return unittest.TestSuite(
        [Suite(os.path.basename(filename),
               optionflags=OPTIONFLAGS,
               package='Products.Collage.tests',
               test_class=CollageFunctionalTestCase)
         for filename in list_doctests()]
        )



# $Id: skins.py 69655 2008-08-08 16:58:31Z glenfant $

from Products.Collage.utilities import CollageMessageFactory as _

class PortletDefaultSkin(object):
    title = _(u"default", default=u"Default")

class PortletNotificationSkin(object):
    title = _(u"notification_skin", default=u"Notification")

class PortletHelpSkin(object):
    title = _(u"help_skin", default=u"Help")



# $Id: test_testutils.py 69655 2008-08-08 16:58:31Z glenfant $
"""
Testing... the test framework
"""

from Products.Collage.tests.base import CollageTestCase
from Products.Collage.tests import utils as ctc_utils

class UtilsTestCase(CollageTestCase):
    """We test utilities for testcases"""

    def testTestRequest(self):
        request = ctc_utils.TestRequest()
        request.set('dummy', 'stuff')
        self.failUnlessEqual(request.get('dummy'), 'stuff')
        return

    def testAddcollage(self):
        self.loginAsPortalOwner()
        foo_collage = ctc_utils.addCollage(self.portal, 'foo', 'Foo')
        self.failUnlessEqual(foo_collage.title_or_id(), 'Foo')
        return

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(UtilsTestCase))
    return suite



# -*- coding: utf-8 -*-
# $Id$
import os
from setuptools import setup, find_packages

def _textOfModuleFile(filename):
    return open(filename, 'r').read().strip()

version = _textOfModuleFile(os.path.join('Products', 'Collage', 'version.txt'))

setup(name='Products.Collage',
      version=version,
      description=("A product to create page compositions in Plone."),
      long_description='\n\n'.join([
          _textOfModuleFile(name)
          for name in (
               'README.txt',
               os.path.join('docs', 'DEVELOPERS.txt'),
               os.path.join('docs', 'HISTORY.txt'))]),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Plone",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Development Status :: 5 - Production/Stable",
        ],
      keywords='plone layout composition',
      author='Malthe Borch',
      author_email='mborch@gmail.com',
      url='http://www.plone.org/products/collage',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'Products.Archetypes',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
# $Id: utilities.py 82661 2009-03-18 07:33:10Z jensens $
"""Misc utilities"""

import re

isNumber = re.compile(r"^\d+$")

def findFirstAvailableInteger(ids):
    i = 1
    while True:
        if str(i) not in ids:
            return i
        i += 1

def generateNewId(container):
    parent_contents = container.objectValues()
    contentIDs = map(lambda x: x.getId(), parent_contents)
    numericalIDs = filter(isNumber.match, contentIDs)
    return str(findFirstAvailableInteger(numericalIDs))


###
## Our i18n message factory
###

from zope.i18nmessageid import MessageFactory
from Products.Collage.config import I18N_DOMAIN

CollageMessageFactory = MessageFactory(I18N_DOMAIN)

###
## Detects translatable objects when LP installed
###

# FIXME: Should we check that LP is installed in this site too ?
try:
    from Products.LinguaPlone.interfaces import ITranslatable
except ImportError:
    HAS_LINGUAPLONE = False
else:
    HAS_LINGUAPLONE = True

def isTranslatable(content):
    if HAS_LINGUAPLONE:
        return ITranslatable.providedBy(content)
    return False

###
## Logging utility
###

from Products.Collage.config import PROJECTNAME
import logging
logger = logging.getLogger(PROJECTNAME)

###
## Get the portal object without context/request
###

from zope.component import getUtility
from Products.CMFCore.interfaces import ISiteRoot

def getPortal():
    return getUtility(ISiteRoot)

###
## Get Collage site options
###

from zope.component import getAdapter
from Products.Collage.interfaces import ICollageSiteOptions

def getCollageSiteOptions():
    """Collage site options from contol panel"""

    return getAdapter(getPortal(), ICollageSiteOptions)

###
## Version of Collage
## Other components that rely on Collage (skinning, templating, ...) may need this
###

from Products.Collage.config import PACKAGE_HOME
from Products.CMFPlone.utils import versionTupleFromString

def getFSVersionTuple():
    """Reads version.txt and returns version tuple"""
    vfile = "%s/version.txt" % PACKAGE_HOME
    v_str = open(vfile, 'r').read().lower().strip()
    return versionTupleFromString(v_str)

###
## Upgrade steps decorator
###

# Background: GenericSetup shows upgrade steps for components that are
# *not* installed in the site, and let the Manager execute these
# upgrade steps. This is somehow harmful. While this bug is not fixed,
# this safety belt will prevent managers executing the exposed upgrade
# steps.
# See https://dev.plone.org/plone/ticket/8507
# Usage:
#
#  @IfInstalled('Collage')
#  def someUpgradeScript(setuptool):
#      # Usual upgrade script

class NotInstalledComponent(LookupError):
    def __init__(self, cpt_name):
        self.cpt_name = cpt_name
        return

    def __str__(self):
        msg = ("Component '%s' is not installed in this site."
               " You can't run its upgrade steps."
               % self.cpt_name)
        return msg

class IfInstalled(object):
    def __init__(self, prod_name=PROJECTNAME):
        """@param prod_name: as shown in quick installer"""
        self.prod_name = prod_name

    def __call__(self, func):
        """@param func: the decorated function"""
        def wrapper(setuptool):
            qi = getPortal().portal_quickinstaller
            installed_ids = [p['id'] for p in qi.listInstalledProducts()]
            if self.prod_name not in installed_ids:
                raise NotInstalledComponent(self.prod_name)
            return func(setuptool)
        wrapper.__name__ = func.__name__
        wrapper.__dict__.update(func.__dict__)
        wrapper.__doc__ = func.__doc__
        wrapper.__module__ = func.__module__
        return wrapper




# -*- coding: utf-8 -*-
# $Id: views.py 145424 2010-10-04 12:25:02Z khink $

from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import ComponentLookupError
from zope.interface import providedBy
from zope.interface import Interface
from plone.memoize.view import memoize_contextless
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.Collage.interfaces import IDynamicViewManager, IPortletSkin
from Products.Collage.interfaces import ICollageBrowserLayer
from Products.Collage.utilities import CollageMessageFactory as _
from Products.Collage.utilities import getCollageSiteOptions

def test(condition, value_true, value_false):
    if condition:
        return value_true
    else:
        return value_false

class BaseView(BrowserView):
    hide = False
    __alias__ = None

    def test(self):
        return test

    @memoize_contextless
    def isAnon(self):
        return self.mtool().isAnonymousUser()

    @memoize_contextless
    def normalizeString(self):

        return getToolByName(self.context, 'plone_utils').normalizeString

    @memoize_contextless
    def mtool(self):

        plone_tools = getMultiAdapter((self.context, self.request), name=u'plone_tools')
        return plone_tools.membership()

    @memoize_contextless
    def portal_url(self):

        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        return portal_state.portal_url()

    @memoize_contextless
    def site_properties(self):

        plone_tools = getMultiAdapter((self.context, self.request), name=u'plone_tools')
        return plone_tools.properties().site_properties

    @memoize_contextless
    def friendlyTypes(self):

        return getToolByName(self.context, 'plone_utils').getUserFriendlyTypes()

    @property
    def collage_context(self):
        alias = getattr(self, '__alias__', None)

        if alias:
            return alias

        return self.__parent__

    def getSkin(self):
        manager = IDynamicViewManager(self.collage_context)
        return manager.getSkin()

class ErrorViewNotFoundView(BaseView):
    title = u'View not Found'
    hide = True
    notfoundlayoutname = None

class RowView(BaseView):

    def getColumnBatches(self, bsize=None):
        """Rows with more than *bsize* columns are split.

        @param bsize: number of max. allowed columns per row. 0 for no batching.

        @return: list of columns, each containing a list of rows.
        """
        if not bsize:
            options = getCollageSiteOptions()
            if not hasattr(options, 'batch_size'):
                bsize = 3
            else:
                bsize = options.batch_size

        columns = self.context.folderlistingFolderContents()
        if not columns:
            return []
        if bsize == 0:
            return [columns,]
        numbatches = (len(columns) - 1) / bsize + 1
        batches = []
        for numbatch in range(numbatches):
            batch = []
            for bidx in range(bsize):
                index = numbatch * bsize + bidx
                column = None
                if index < len(columns):
                    column = columns[index]
                # pad with null-columns, but do not pad first row
                if column or numbatch > 0:
                    batch.append(column)
            batches.append(batch)
        return batches

class AutomaticRowView(RowView):
    title = _(u'Automatic')

class LargeLeftRowView(RowView):
    title = _(u'Large left')

class LargeRightRowView(RowView):
    title = _(u'Large right')

class UnbatchedRowView(RowView):
    title = _(u'Unbatched')

class StandardView(BaseView):
    title = _(u'Standard')

class TextView(BaseView):
    title = _(u'Text')

class FeaturedView(BaseView):
    title = _(u'Featured')

class PortletView(BaseView):
    title = _(u'Portlet')
    skinInterfaces = (IPortletSkin,)

class AlbumTopicView(BaseView):
    title = _(u'Album')

class SummaryTopicView(BaseView):
    title = _(u'Summary')

class TabularTopicView(BaseView):
    title = _(u'Tabular')

class InheritTopicView(BaseView):
    """Inherits view from topic's display setting."""

    title = _(u'Inherit')

    mapping = {
        'folder_listing': 'standard',
        'folder_summary_view': 'summary',
        'folder_tabular_view': 'tabular',
        'atct_album_view': 'album',
        'atct_topic_view': 'standard',
        }

    error_view_name = "error_collage-view-not-found"
    fallback_view_name = "fallback"

    def __call__(self):
        """Determine inherited view and attempt to find suitable
        collage view.

        If a view can't be determined, render fallback view. If the
        view does not exist, render an error message.
        """

        layout = self.context.getLayout()
        name = self.mapping.get(layout, self.fallback_view_name)
        spec = providedBy(self.context), ICollageBrowserLayer
        lookup = getSiteManager().adapters.lookup
        factory = lookup(spec, Interface, name=name)
        if factory is None:
            name = None
            factory = lookup(spec, Interface, name=self.error_view_name)
            if factory is None:
                raise ComponentLookupError(
                    "Layout not found: %s (and unable to render error view)." % \
                    layout)
        view = factory(self.context, self.request)
        if name is None:
            view.notfoundlayoutname = layout

        view.__alias__ = self.__alias__
        return view()

class ClickableView(BaseView):
    title = _(u'Clickable')

class StandardDocumentView(StandardView):
    """Includes for BBB."""

class FileMinimalView(StandardView):
    """File for download in one line."""

    title = _(u'minimal')

    def getBUFile(self):
        acc = self.context.Schema()['file'].getAccessor(self.context)()
        return acc
